@@ -539,6 +539,55 @@ class TestModActions(ActionsTest):
         yml = decl.dump(change_set.actions)
         decl.apply(migration_model, io.StringIO(yml))
 
+    def test_identifier_changes_result_in_extensions(
+        self, migration_model: capellambse.MelodyModel
+    ) -> None:
+        tracker = copy.deepcopy(self.tracker)
+        tracker["data_types"]["type1"] = tracker["data_types"]["type"]
+        del tracker["data_types"]["type"]
+        sysr_attr_defs = tracker["requirement_types"]["system_requirement"][
+            "attributes"
+        ]
+        sysr_attr_defs["type1"] = sysr_attr_defs["type"]
+        del sysr_attr_defs["type"]
+        soft_attr_defs = tracker["requirement_types"]["software_requirement"][
+            "attributes"
+        ]
+        soft_attr_defs["type1"] = soft_attr_defs["type"]
+        del soft_attr_defs["type"]
+        item_attrs = tracker["items"][0]["attributes"]
+        item_attrs["type1"] = item_attrs["type"]
+        del item_attrs["type"]
+        item_attrs = tracker["items"][0]["children"][0]["attributes"]
+        item_attrs["type1"] = item_attrs["type"]
+        del item_attrs["type"]
+        item_attrs = tracker["items"][1]["children"][0]["attributes"]
+        item_attrs["type1"] = item_attrs["type"]
+        del item_attrs["type"]
+
+        change_set = self.tracker_change(
+            migration_model, tracker, gather_logs=True
+        )
+
+        edefs = migration_model.search("EnumerationDataTypeDefinition")
+        assert edefs.by_identifier("type"), "Data type definition missing"
+        adefs = migration_model.search("AttributeDefinitionEnumeration")
+        for id in ("system_requirement", "software_requirement"):
+            assert f"type {id}" in adefs.by_identifier  # type: ignore [operator]
+        assert change_set and change_set.actions
+
+        yml = decl.dump(change_set.actions)
+        decl.apply(migration_model, io.StringIO(yml))
+
+        edefs = migration_model.search("EnumerationDataTypeDefinition")
+        adefs = migration_model.search("AttributeDefinitionEnumeration")
+
+        assert not edefs.by_identifier("type")
+        assert edefs.by_identifier("type1")
+        for id in ("system_requirement", "software_requirement"):
+            assert f"type {id}" not in adefs.by_identifier  # type: ignore [operator]
+            assert f"type1 {id}" in adefs.by_identifier  # type: ignore [operator]
+
     @pytest.mark.integtest
     def test_calculate_change_sets(
         self, migration_model: capellambse.MelodyModel
@@ -566,10 +615,8 @@ class TestDeleteActions(ActionsTest):
     REQ_DEL, FOLDER_DEL = TEST_MODULE_CHANGE_2[-2:]
 
     def resolve_ChangeSet(
-        self,
-        model: capellambse.MelodyModel,
-        change_set: cabc.MutableMapping[str, t.Any],
-    ) -> cabc.MutableMapping[str, t.Any]:
+        self, model: capellambse.MelodyModel, change_set: dict[str, t.Any]
+    ) -> dict[str, t.Any]:
         """Fix promised objects in the ``ChangeSet``.
 
         Some objects are created dynamically in the ``deletion_model``
@@ -601,7 +648,9 @@ class TestDeleteActions(ActionsTest):
             "requirement_types"
         ]
         snapshot["items"] = []
-        data_type_del = copy.deepcopy(self.REQ_TYPE_FOLDER_DEL)
+        data_type_del: dict[str, t.Any] = copy.deepcopy(
+            self.REQ_TYPE_FOLDER_DEL
+        )
         del data_type_del["delete"]["requirement_types"]
         data_type_del = self.resolve_ChangeSet(deletion_model, data_type_del)
         expected_actions = [self.ENUM_DATA_TYPE_DEL, data_type_del]
@@ -620,7 +669,7 @@ class TestDeleteActions(ActionsTest):
         snapshot["items"] = []
         req_type_del = copy.deepcopy(self.REQ_TYPE_FOLDER_DEL)
         del req_type_del["delete"]["data_type_definitions"]
-        attr_def_del = copy.deepcopy(self.ATTR_DEF_DEL)
+        attr_def_del: dict[str, t.Any] = copy.deepcopy(self.ATTR_DEF_DEL)
         attr_def_del = self.resolve_ChangeSet(deletion_model, attr_def_del)
 
         tchange = self.tracker_change(deletion_model, snapshot)
@@ -647,7 +696,9 @@ class TestDeleteActions(ActionsTest):
         self, deletion_model: capellambse.MelodyModel
     ) -> None:
         """Test ChangeSet on clean model for first migration run."""
-        expected_change_set = copy.deepcopy(TEST_MODULE_CHANGE_2)
+        expected_change_set: list[dict[str, t.Any]] = copy.deepcopy(
+            TEST_MODULE_CHANGE_2
+        )
         expected_change_set[1] = self.resolve_ChangeSet(
             deletion_model, TEST_MODULE_CHANGE_2[1]
         )
@@ -681,7 +732,7 @@ class TestCalculateChangeSet(ActionsTest):
         del config["capella-uuid"]  # type:ignore[misc]
         message = (
             "The given module configuration is missing UUID of the target "
-            "RequirementsModule"
+            "CapellaModule"
         )
 
         with caplog.at_level(logging.ERROR):
@@ -703,7 +754,7 @@ class TestCalculateChangeSet(ActionsTest):
         tconfig = TEST_CONFIG["modules"][0]
         del clean_model.la.requirement_modules[0]
         message = (
-            f"No RequirementsModule with UUID '{TEST_REQ_MODULE_UUID}' found "
+            f"No CapellaModule with UUID '{TEST_REQ_MODULE_UUID}' found "
             f"in {clean_model.info!r}"
         )
 
@@ -745,7 +796,7 @@ class TestCalculateChangeSet(ActionsTest):
         del config["capella-uuid"]  # type:ignore[misc]
         message = (
             "The given module configuration is missing UUID of the target "
-            "RequirementsModule"
+            "CapellaModule"
         )
 
         _, errors = calculate_change_set(
